#include <QtGui>
#include "windowplay.h"

#define phtonl(p, v) \
		{                       \
		((unsigned char*)(p))[0] = (unsigned char)((v) >> 24);  \
		((unsigned char*)(p))[1] = (unsigned char)((v) >> 16);  \
		((unsigned char*)(p))[2] = (unsigned char)((v) >> 8);   \
		((unsigned char*)(p))[3] = (unsigned char)((v) >> 0);   \
		}

#define phtons(p, v) \
	{ 				\
	(p)[0] = (unsigned char)((v) >> 8);	\
	(p)[1] = (unsigned char)((v) >> 0);	\
	}

#define	SEGMENT_SHIFT	(4)		/* Left shift. */
#define	SIGN_BIT_LAW	(0x80)		/* Sign bit for an A-law byte in the packet. */
#define	BIAS_CODE		(0x84)		/* Bias for linear code. */
#define	SEGMENT_MASK	(0x70)		/* Segment mask field. */
#define	NUM_SEGS		(8)			/* Number of A-law segments. */
#define	QUANT_MASK_FIELD	(0xf)		/* Quantization mask field. */

int ulaw2linear(unsigned char	val)
	{
	int		t;

	val = ~val;

	t = ((val & QUANT_MASK_FIELD) << 3) + BIAS_CODE;
	t <<= ((unsigned)val & SEGMENT_MASK) >> SEGMENT_SHIFT;

	return ((val & SIGN_BIT_LAW) ? (BIAS_CODE - t) : (t - BIAS_CODE));
	}

int alaw2linear(unsigned char	val)
	{
	int		t;
	int		seg;

	val ^= 0x55;

	t = (val & QUANT_MASK_FIELD) << 4;
	seg = ((unsigned)val & SEGMENT_MASK) >> SEGMENT_SHIFT;
	switch (seg)
		{
		case 0:
			{
			t += 8;
			break;
			}
		case 1:
			{
			t += 0x108;
			break;
			}
		default:
			{
			t += 0x108;
			t <<= seg - 1;
			}
		}
	return ((val & SIGN_BIT_LAW) ? t : -t);
	}

WindowPlay::WindowPlay()
	{
	setWindowTitle(tr("Reprodutor"));
	setAttribute(Qt::WA_DeleteOnClose);

	QSize buttonSize(34, 28);

	rewindButton = new QPushButton(this);
	rewindButton->setIcon(style()->standardIcon(QStyle::SP_MediaSkipBackward));

	playButton = new QPushButton(this);
	playIcon = style()->standardIcon(QStyle::SP_MediaPlay);
	pauseIcon = style()->standardIcon(QStyle::SP_MediaPause);
	playButton->setIcon(playIcon);

	slider = new Phonon::SeekSlider(this);
	slider->setMediaObject(&m_MediaObject);
	volume = new Phonon::VolumeSlider(&m_AudioOutput);

	QVBoxLayout *vLayout = new QVBoxLayout(this);
	vLayout->setContentsMargins(8, 8, 8, 8);

	QHBoxLayout *layout = new QHBoxLayout();

	rewindButton->setMinimumSize(buttonSize);
	playButton->setMinimumSize(buttonSize);

	volume->setFixedWidth(100);

	layout->addWidget(rewindButton);
	layout->addWidget(playButton);

	layout->addStretch();
	layout->addWidget(volume);

	QVBoxLayout *buttonPanelLayout = new QVBoxLayout();
	buttonPanelLayout->addLayout(layout);

	timeLabel = new QLabel(this);
	progressLabel = new QLabel(this);
	QWidget *sliderPanel = new QWidget(this);
	QHBoxLayout *sliderLayout = new QHBoxLayout();
	sliderLayout->addWidget(slider);
	sliderLayout->addWidget(timeLabel);
	sliderLayout->addWidget(progressLabel);
	sliderLayout->setContentsMargins(0, 0, 0, 0);
	sliderPanel->setLayout(sliderLayout);

	buttonPanelLayout->addWidget(sliderPanel);
	buttonPanelLayout->setContentsMargins(0, 0, 0, 0);

	layout->setSpacing(4);
	buttonPanelLayout->setSpacing(0);

	QWidget *buttonPanelWidget = new QWidget(this);
	buttonPanelWidget->setSizePolicy(QSizePolicy::Preferred, QSizePolicy::Fixed);
	buttonPanelWidget->setLayout(buttonPanelLayout);
	vLayout->addWidget(buttonPanelWidget);

	QWidget *widget = new QWidget();
	widget->setLayout(vLayout);

	setCentralWidget(widget);

	// Setup signal connections:
	connect(rewindButton, SIGNAL(clicked()), this, SLOT(rewind()));
	connect(playButton, SIGNAL(clicked()), this, SLOT(playPause()));
	connect(&m_MediaObject, SIGNAL(totalTimeChanged(qint64)), this, SLOT(updateTime()));
	connect(&m_MediaObject, SIGNAL(tick(qint64)), this, SLOT(updateTime()));
	connect(&m_MediaObject, SIGNAL(stateChanged(Phonon::State,Phonon::State)), this, SLOT(stateChanged(Phonon::State,Phonon::State)));
	connect(&m_MediaObject, SIGNAL(bufferStatus(int)), this, SLOT(bufferStatus(int)));

	rewindButton->setEnabled(false);
	playButton->setEnabled(false);

	m_audioOutputPath = Phonon::createPath(&m_MediaObject, &m_AudioOutput);
	resize(minimumSizeHint());
	}

WindowPlay::~WindowPlay(void)
   {
   remove("temp.au");
   }

void WindowPlay::rewind()
	{
	m_MediaObject.seek(0);
	}

void WindowPlay::playPause()
	{
	if (m_MediaObject.state() == Phonon::PlayingState)
		m_MediaObject.pause();
	else
		{
		if (m_MediaObject.currentTime() == m_MediaObject.totalTime())
			m_MediaObject.seek(0);
		m_MediaObject.play();
		}
	}

void WindowPlay::updateTime()
	{
	long len = m_MediaObject.totalTime();
	long pos = m_MediaObject.currentTime();
	QString timeString;
	if (pos || len)
		{
		int sec = pos/1000;
		int min = sec/60;
		int hour = min/60;
		int msec = pos;

		QTime playTime(hour%60, min%60, sec%60, msec%1000);
		sec = len / 1000;
		min = sec / 60;
		hour = min / 60;
		msec = len;

		QTime stopTime(hour%60, min%60, sec%60, msec%1000);
		QString timeFormat = "m:ss";
		if (hour > 0)
			timeFormat = "h:mm:ss";
		timeString = playTime.toString(timeFormat);
		if (len)
			timeString += " / " + stopTime.toString(timeFormat);
		}
	timeLabel->setText(timeString);
	}

void WindowPlay::stateChanged(Phonon::State newstate, Phonon::State oldstate)
	{
	Q_UNUSED(oldstate);

	switch (newstate)
		{
		case Phonon::ErrorState:
			{
			if (m_MediaObject.errorType() == Phonon::FatalError)
				{
				playButton->setEnabled(false);
				rewindButton->setEnabled(false);
				}
			else
				{
				m_MediaObject.pause();
				}
			QMessageBox::warning(this, "Phonon Mediaplayer", m_MediaObject.errorString(), QMessageBox::Close);
			break;
			}
	  case Phonon::StoppedState:
	  case Phonon::PausedState:
			{
			playButton->setIcon(playIcon);
			if (m_MediaObject.currentSource().type() != Phonon::MediaSource::Invalid)
				{
				playButton->setEnabled(true);
				rewindButton->setEnabled(true);
				}
			else
				{
				playButton->setEnabled(false);
				rewindButton->setEnabled(false);
				}
			break;
			}
	  case Phonon::PlayingState:
			{
			playButton->setEnabled(true);
			playButton->setIcon(pauseIcon);
			}
	  case Phonon::BufferingState:
			{
			rewindButton->setEnabled(true);
			break;
			}
	  case Phonon::LoadingState:
			{
			rewindButton->setEnabled(false);
			break;
			}
		}
	}

void WindowPlay::bufferStatus(int percent)
	{
	if (percent == 0 || percent == 100)
		progressLabel->setText(QString());
	else
		{
		QString str = QString::fromLatin1("(%1%)").arg(percent);
		progressLabel->setText(str);
		}
	}

void WindowPlay::closeEvent(QCloseEvent *event)
	{
	remove("temp.au");
	emit closingPlay(true);
	}

void WindowPlay::setInfoPlay(stream_RTP_t streamRTP)
	{
	FILE *to_stream;
	int tamOrg;
	int tamRec;
	int j;
	unsigned char pd[4];
	short sample;
	size_t nchars;

	tamOrg = streamRTP.originator.size();
	tamRec = streamRTP.receiver.size();

	if ((streamRTP.codecOriginator == G711_PCMA) || (streamRTP.codecOriginator == G711_PCMU))
		{
		to_stream = fopen("temp.au", "wb");
		phtonl(pd, 0x2e736e64);
		nchars=fwrite(pd, 1, 4, to_stream);
		/* header offset == 24 bytes */
		phtonl(pd, 24);
		nchars=fwrite(pd, 1, 4, to_stream);
		/* total length; it is permitted to set this to 0xffffffff */
		phtonl(pd, -1);
		nchars=fwrite(pd, 1, 4, to_stream);
		/* encoding format == 16-bit linear PCM */
		phtonl(pd, 3);
		nchars=fwrite(pd, 1, 4, to_stream);
		/* sample rate == 8000 Hz */
		phtonl(pd, 8000);
		nchars=fwrite(pd, 1, 4, to_stream);
		/* channels == 1 */
		phtonl(pd, 1);
		nchars=fwrite(pd, 1, 4, to_stream);

		if (streamRTP.codecOriginator == G711_PCMA)
			{
			for (j = 0; j < tamOrg; j++)
				{
				sample = alaw2linear((unsigned char)streamRTP.originator.at(j));
				if (j < tamRec)
					{
					sample = (sample + alaw2linear((unsigned char)streamRTP.receiver.at(j))) / 2;
					}
				else
					{
					sample = (sample + SILENCIO_PCMA) / 2;
					}
				phtons(pd, sample);
				fwrite(pd, 1, 2, to_stream);
				}
			fclose(to_stream);
			}
		else if (streamRTP.codecOriginator == G711_PCMU)
			{
			for (j = 0; j < tamOrg; j++)
				{
				sample = ulaw2linear((unsigned char)streamRTP.originator.at(j));
				if (j < tamRec)
					{
					sample = (sample + ulaw2linear((unsigned char)streamRTP.receiver.at(j))) / 2;
					}
				else
					{
					sample = (sample + SILENCIO_PCMU) / 2;
					}

				phtons(pd, sample);
				fwrite(pd, 1, 2, to_stream);
				}
			}
		fclose(to_stream);
		m_MediaObject.clearQueue();
		m_MediaObject.enqueue(Phonon::MediaSource("temp.au"));
		}
	else
		{
		QMessageBox msgBox;
		msgBox.setText(tr("Codificador de udio no suportado"));
		msgBox.exec();
		}
	}

